/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/*
==========================================================================

						 - SPLITMODELS -

==========================================================================
*/
// - Adding the Player model using Skeletal animation blending
// by Jalisk0

#include "cg_local.h"
#include "../game/gs_pmodels.h"


//======================================================================
//						PlayerModel Registering
//======================================================================

pmodel_t		cg_entPModels[MAX_EDICTS];
pmodelinfo_t	cg_PModelInfos[CG_MAX_PMODELS];
pskin_t			cg_pSkins[CG_MAX_PSKINS];



/*
================
CG_PModelsInit

  link PModel handles to it's cg_entities
================
*/
void CG_PModelsInit(void)
{
	memset(cg_entPModels, 0, sizeof(cg_entPModels));
}


/*
================
CG_CopyAnimation
================
*/
void CG_CopyAnimation (pmodelinfo_t *pmodelinfo, int put, int pick)
{
	pmodelinfo->firstframe[put] = pmodelinfo->firstframe[pick];
	pmodelinfo->lastframe[put] = pmodelinfo->lastframe[pick];
	pmodelinfo->loopingframes[put] = pmodelinfo->loopingframes[pick];
}


//================
//CG_FindBoneNum
//================
int CG_FindBoneNum( cgs_skeleton_t *skel, char *bonename)
{
	int	j;

	if( !skel )
		return -1;

	for( j = 0; j < skel->numBones; j++ ) 
	{
		if( !strcmp(skel->bones[j].name, bonename ) )
			return j;
	}

	return -1;
}

//================
//CG_ParseRotationBone
//================
void CG_ParseRotationBone(pmodelinfo_t *pmodelinfo, char *token, int pmpart)
{
	int boneNumber;
	
	boneNumber = CG_FindBoneNum( CG_SkeletonForModel( pmodelinfo->model ), token);
	if(boneNumber < 0) {
		if(cg_debugPlayerModels->integer)
			CG_Printf("CG_ParseRotationBone: No such bone name %s\n", token);
		return;
	}
	//register it into pmodelinfo
	CG_Printf("Script: CG_ParseRotationBone: %s is %i\n", token, boneNumber);
	pmodelinfo->rotator[pmpart][pmodelinfo->numRotators[pmpart]] = boneNumber;
	pmodelinfo->numRotators[pmpart]++;
}

//================
//CG_ParseTagMask
//================
void CG_ParseTagMask(pmodelinfo_t *pmodelinfo, int bonenum, char *name)
{
	cg_tagmask_t *tagmask;
	cgs_skeleton_t	*skel;

	if( !name || !name[0] )
		return;

	skel = CG_SkeletonForModel( pmodelinfo->model );
	if( !skel || skel->numBones <= bonenum ) 
		return;

	//fixme: check the name isn't already in use, or it isn't the same as a bone name

	//now store it
	tagmask = CG_Malloc( sizeof(cg_tagmask_t));
	Q_snprintfz( tagmask->tagname, sizeof(tagmask->tagname), name );
	Q_snprintfz( tagmask->bonename, sizeof(tagmask->bonename), skel->bones[bonenum].name );
	tagmask->bonenum = bonenum;
	tagmask->next = pmodelinfo->tagmasks;
	pmodelinfo->tagmasks = tagmask;
}

/*
================
CG_ParseAnimationScript

  Reads the animation config file.

0 = first frame
1 = number of frames/lastframe
2 = looping frames
3 = frame time
  
Note: The animations count begins at 1, not 0. I preserve zero for "no animation change"
---------------
New keywords:
nooffset: Uses the first frame value at the script, and no offsets, for the animation.
alljumps: Uses 3 different jump animations (bunnyhoping)
islastframe: second value of each animation is lastframe instead of numframes
	
Q3 keywords:
sex m/f : sets gender
	  
Q3 Unsupported:
headoffset <value> <value> <value>
footsteps
================
*/
qboolean CG_ParseAnimationScript ( pmodelinfo_t *pmodelinfo, char *filename )
{
	qbyte		*buf;
	char		*ptr, *token;
	int			rounder, counter, i, offset;
	qboolean	lower_anims_have_offset = qfalse;
	qboolean	alljumps = qfalse;
	qboolean	islastframe = qtrue;
	qboolean	debug = qtrue;
	int			anim_data[3][PMODEL_MAX_ANIMS];//splitskel:data is: firstframe, lastframe, loopingframes
	int			filenum;
	int			length;

	pmodelinfo->tagmasks = NULL;
	pmodelinfo->sex = GENDER_MALE;
	pmodelinfo->frametime = 1000/24;	//24fps by default
	rounder = 0;
	counter = 1; //reseve 0 for 'no animation'

	if (!cg_debugPlayerModels->integer)
		debug = qfalse;

	// load the file
	length = trap_FS_FOpenFile( filename, &filenum, FS_READ );
	if( length == -1 ) {
		CG_Printf ("Couldn't load animation script: %s\n", filename);
		return qfalse;
	}
	if( !length ) {
		trap_FS_FCloseFile( filenum );
		return qfalse;
	}
	buf = CG_Malloc( length + 1 );
	trap_FS_Read( buf, length, filenum );
	trap_FS_FCloseFile( filenum );
	
	if( !buf ) {
		CG_Free(buf);
		CG_Printf ("Couldn't load animation script: %s\n", filename);
		return qfalse;
	}
	
	//proceed
	ptr = ( char * )buf;
	while ( ptr )
	{
		token = COM_ParseExt ( &ptr, qtrue );
		if ( !token )
			break;
		
		if (*token < '0' || *token > '9') {
			
			//gender
			if ( !Q_stricmp (token, "sex") ) {
				
				if (debug)
					CG_Printf ("Script: %s:", token);
				
				token = COM_ParseExt ( &ptr, qfalse );
				if ( !token )	//Error (fixme)
					break;
				
				if ( token[0] == 'm' || token[0] == 'M' ) {
					pmodelinfo->sex = GENDER_MALE;
						if (debug)
							CG_Printf (" %s -Gender set to MALE\n", token);
					
				} else if ( token[0] == 'f' || token[0] == 'F' ) {
					
					pmodelinfo->sex = GENDER_FEMALE;
						if (debug)
							CG_Printf (" %s -Gender set to FEMALE\n", token);
				} else if ( token[0] == 'n' || token[0] == 'N' ) {
					pmodelinfo->sex = GENDER_NEUTRAL;
						if (debug)
							CG_Printf (" %s -Gender set to NEUTRAL\n", token);
				} else {
					if (debug){
						if(token[0])
							CG_Printf (" WARNING: unrecognized token: %s\n", token);
						else
							CG_Printf (" WARNING: no value after cmd sex: %s\n", token);
					}
					break; //Error
				}
				
			//nooffset
			} else if ( !Q_stricmp (token, "offset") ) {
				lower_anims_have_offset = qtrue;
				if (debug)
					CG_Printf ("Script: Using offset values for lower frames\n");

			//alljumps
			} else if ( !Q_stricmp (token, "alljumps") ) {
				alljumps = qtrue;
				if (debug)
					CG_Printf ("Script: Using all jump animations\n");

			//islastframe
			} else if ( !Q_stricmp (token, "isnumframes") ) {
				islastframe = qfalse;
				if (debug)
					CG_Printf ("Script: Second value is read as numframes\n");

			} else if ( !Q_stricmp (token, "islastframe") ) {
				islastframe = qtrue;
				if (debug)
					CG_Printf ("Script: Second value is read as lastframe\n");

			//FPS
			} else if ( !Q_stricmp (token, "fps") || !Q_stricmp (token, "FPS") ) {
				int fps;

				token = COM_ParseExt ( &ptr, qfalse );
				if ( !token )	//Error (fixme)
					break;

				fps = (int)atoi(token);
				if(fps == 0 && debug)	//Error (fixme)
					CG_Printf ("Script: WARNING:Invalid FPS value (%s) in config for playermodel %s \n", token, filename );

				if(fps < 10)	//never allow less than 10 fps
					fps = 10;

				pmodelinfo->frametime = ( 1000/(float)fps );
				if (debug)
					CG_Printf ("Script: FPS: %i\n", fps);

			//boneKeys
			} else if ( !Q_stricmp (token, "bonekeys") ) {
				int j;

				token = COM_ParseExt ( &ptr, qfalse );
				if ( !token )	//Error (fixme)
					break;

				pmodelinfo->numbonekeys = (int)atoi(token);
				if( pmodelinfo->numbonekeys == 0 || pmodelinfo->numbonekeys > PMODEL_PARTS ) {	//Error (fixme)
					CG_Printf ("Script: ERROR: Wrong numbonekeys value: %s\n", token);
					break;
				} else if(debug)
					CG_Printf ("Script: numbonekeys: %s keys: ", token);

				for( j=0; j<pmodelinfo->numbonekeys; j++)
				{
					token = COM_ParseExt ( &ptr, qfalse );
					if(!token || strlen(token) > 1){
						CG_Printf ("\nERROR: Wrong bonekey: %s \n", token);
						CG_Free(buf);
						return qfalse;	//Error (fixme)
					} else if(debug)
						CG_Printf ("%s ", token);

					pmodelinfo->boneKeys[j] = token[0];
				}

				if(debug)
					CG_Printf ("\n");

			//Rotation bone
			} else if ( !Q_stricmp (token, "rotationbone") ) {
				token = COM_ParseExt ( &ptr, qfalse );
				if(!token || !token[0])
					break;	//Error (fixme)

				if ( !Q_stricmp (token, "upper") ) {
					token = COM_ParseExt ( &ptr, qfalse );
					if(!token || !token[0])
						break;	//Error (fixme)
					CG_ParseRotationBone(pmodelinfo, token, UPPER);
				}
				else if ( !Q_stricmp (token, "head") ) {
					token = COM_ParseExt ( &ptr, qfalse );
					CG_ParseRotationBone(pmodelinfo, token, HEAD);
				}
				else {
					if(debug)
					CG_Printf ("Script: ERROR: Unrecognized rotation pmodel part %s\n", token);
					CG_Printf ("Script: ERROR: Valid names are: 'upper', 'head'\n");
				}
			
			//Tag bone (format is: tagmask "bone name" "tag name")
			} else if ( !Q_stricmp (token, "tagmask") ) {
				int	bonenum;

				token = COM_ParseExt ( &ptr, qfalse );
				if(!token || !token[0])
					break;//Error

				bonenum =  CG_FindBoneNum( CG_SkeletonForModel( pmodelinfo->model ), token);
				if( bonenum != -1 ) {
					//store the tag mask
					CG_ParseTagMask(pmodelinfo, bonenum, COM_ParseExt ( &ptr, qfalse ) );
				} else {
					if(debug)
						CG_Printf ("Script: WARNING: Unknown bone name: %s\n", token);
				}

			} else if (token[0] && debug)
				CG_Printf ("Script: WARNING: unrecognized token: %s\n", token);

			
		} else {
			
			//frame & animation values
			i = (int)atoi(token);
			if (debug)
				CG_Printf ("%i - ", i);
			anim_data[rounder][counter] = i;
			rounder++;
			if (rounder > 2){
				rounder = 0;
				if (debug)
					CG_Printf (" anim: %i\n", counter);
				counter++;
				if(counter == PMODEL_MAX_ANIMATIONS)
					break;
			}
		}
	}
	
	CG_Free(buf);
	
	//it must contain at least as many animations as a Q3 script to be valid
	if(counter-1 < LEGS_TURN) {
		CG_Printf ("PModel Error: Not enough animations(%i) at animations script: %s\n", counter, filename);
		return qfalse;
	}

	
	//animation ANIM_NONE (0) is always at frame 0, and it's never 
	//received from the game, but just used on the client when none
	//animation was ever set for a model (head).
	
	anim_data[0][ANIM_NONE] = 0;
	anim_data[1][ANIM_NONE]	= 0;
	anim_data[2][ANIM_NONE]	= 1;
	
	//reorganize to make my life easier
	for (i=0 ; i<counter ; i++)
	{
		pmodelinfo->firstframe[i] = anim_data[0][i];

		if (islastframe)
			pmodelinfo->lastframe[i] = anim_data[1][i];
		else
			pmodelinfo->lastframe[i] = ((anim_data[0][i]) + (anim_data[1][i]));

		pmodelinfo->loopingframes[i] = anim_data[2][i];
	}
	
	if (lower_anims_have_offset)
	{
		//find offset (Why did they do this???)  
		offset = pmodelinfo->firstframe[LEGS_CRWALK] - pmodelinfo->firstframe[TORSO_GESTURE];
		
		//Remove offset from the lower & extra animations
		for (i = LEGS_CRWALK; i < counter; ++i){
			pmodelinfo->firstframe[i] -= offset;
			pmodelinfo->lastframe[i] -= offset;
		}
		
		if (debug)
			CG_Printf ("PModel: Fixing offset on lower frames (Q3 format)\n");
	}
	
	//Fix the ranges to fit my looping calculations
	for (i = 0; i < counter; ++i)
	{
		if (pmodelinfo->loopingframes[i])
			pmodelinfo->loopingframes[i] -= 1;
		if (pmodelinfo->lastframe[i])
			pmodelinfo->lastframe[i] -= 1;
	}

	//Register Tag MaskNames (jal: ToDo)



	
	//Alljumps:	 I use 3 jump animations for bunnyhoping
	//animation support. But they will only be loaded as
	//bunnyhoping animations if the keyword "alljumps" is
	//present at the animation.cfg. Otherwise, LEGS_JUMP1
	//will be used for all the jump styles.
	
	if (!alljumps)
	{
		CG_CopyAnimation (pmodelinfo, LEGS_JUMP3, LEGS_JUMP1);
		CG_CopyAnimation (pmodelinfo, LEGS_JUMP3ST, LEGS_JUMP1ST);
		CG_CopyAnimation (pmodelinfo, LEGS_JUMP2, LEGS_JUMP1);
		CG_CopyAnimation (pmodelinfo, LEGS_JUMP2ST, LEGS_JUMP1ST);
	}
	
	//HACK Uncomplete scripts (for Q3 player models)
	//----------------------------------------------------
	counter--;
	
	if (counter < TORSO_RUN)
		CG_CopyAnimation (pmodelinfo, TORSO_RUN, TORSO_STAND);
	if (counter < TORSO_DROPHOLD)
		CG_CopyAnimation (pmodelinfo, TORSO_DROPHOLD, TORSO_STAND);
	if (counter < TORSO_DROP)
		CG_CopyAnimation (pmodelinfo, TORSO_DROP, TORSO_ATTACK2);
	if (counter < TORSO_PAIN1)
		CG_CopyAnimation (pmodelinfo, TORSO_PAIN1, TORSO_STAND2);
	if (counter < TORSO_PAIN2)
		CG_CopyAnimation (pmodelinfo, TORSO_PAIN2, TORSO_STAND2);
	if (counter < TORSO_PAIN3)
		CG_CopyAnimation (pmodelinfo, TORSO_PAIN3, TORSO_STAND2);
	if (counter < TORSO_SWIM)
		CG_CopyAnimation (pmodelinfo, TORSO_SWIM, TORSO_STAND);
	
	if (counter < LEGS_WALKBACK)
		CG_CopyAnimation (pmodelinfo, LEGS_WALKBACK, LEGS_RUNBACK);
	if (counter < LEGS_WALKLEFT)
		CG_CopyAnimation (pmodelinfo, LEGS_WALKLEFT, LEGS_WALKFWD);
	if (counter < LEGS_WALKRIGHT)
		CG_CopyAnimation (pmodelinfo, LEGS_WALKRIGHT, LEGS_WALKFWD);
	if (counter < LEGS_RUNLEFT)
		CG_CopyAnimation (pmodelinfo, LEGS_RUNLEFT, LEGS_RUNFWD);
	if (counter < LEGS_RUNRIGHT)
		CG_CopyAnimation (pmodelinfo, LEGS_RUNRIGHT, LEGS_RUNFWD);
	if (counter < LEGS_SWIM)
		CG_CopyAnimation (pmodelinfo, LEGS_SWIM, LEGS_SWIMFWD);

	return qtrue;
}

/*
================
CG_LoadPModelWeaponSet

  These models are also used for the 1st person view attached weapon
================
*/
void CG_LoadPModelWeaponSet ( pmodel_t *pmodel, char *model_name )
{
	int 		i;

	for (i = 0; i < cgs.numWeaponModels; i++) 
	{
		pmodel->weaponIndex[i] = CG_RegisterWeaponModel ( cgs.weaponModels[i] );
		
		if (!cg_vwep->integer)
			break; // only do weapon 0
	}

	//special case for weapon 0. Must always load the animation script
	if (!pmodel->weaponIndex[0])
		pmodel->weaponIndex[0] = CG_CreateWeaponZeroModel(cgs.weaponModels[0]);
}


/*
================
CG_PSkinUpdateRegistration
================
*/
qboolean CG_PSkinUpdateRegistration ( pskin_t *pSkin, char *filename )
{
	//int 		i;
	char		skin_name[MAX_QPATH];
	char		path[MAX_QPATH];
	char		scratch[MAX_QPATH];

	//extract path & skin_name from filename.
	char *t = NULL;
	char *s;

	//find the word after the last slash
	s = filename;
	t = strstr ( s, "/" );
	while (t)
	{
		s = t+1;
		t = strstr ( s, "/" );
	}
	strcpy(skin_name, s);

	//now remove it from the path
	Q_snprintfz (path, strlen(filename) - strlen(skin_name), "%s", filename);

	//proceed
	Q_snprintfz (scratch, sizeof(scratch), "%s/%s.skin", path, skin_name);
	pSkin->skin = trap_R_RegisterSkinFile (scratch);
	
	//failed: clean up the wrong stuff
	if (!pSkin->skin) {
		pSkin->name[0] = 0;
		pSkin->skin = NULL;
		return qfalse;
	}

	//load icon
	Q_snprintfz (scratch, sizeof(scratch), "%s/icon_%s", path, skin_name);
	pSkin->icon = trap_R_RegisterPic (scratch);

	strcpy (pSkin->name, filename);
	return qtrue;
}


/*
===============
CG_FindPSkinSpot
===============
*/
struct pskin_s *CG_FindPSkinSpot ( char *filename )
{
	int 			i;
	int				freespot = -1;

	for (i=0; i<CG_MAX_PSKINS; i++)
	{
		if (cg_pSkins[i].inuse == qtrue)
		{
			if (!Q_stricmp (cg_pSkins[i].name, filename))//found it
			{
				if (cg_debugPlayerModels->integer)
					Com_Printf ("pSkin: found at spot %i: %s\n", i, filename);

				return &cg_pSkins[i];
			}
		}
		else if (freespot < 0)
			freespot = i;
	}

	if (freespot < 0)
		CG_Error ("CG_FindPSkinsSpot: Couldn't find a free player skin spot");

	if (cg_debugPlayerModels->integer)
		Com_Printf ("pSkin: assigned free spot %i for pSkin %s\n", freespot, filename);

	//we have a free spot
	return &cg_pSkins[freespot];
}


/*
===============
CG_RegisterPSkin
===============
*/
struct pskin_s *CG_RegisterPSkin ( char *filename )
{
	pskin_t	*pSkin;

	COM_StripExtension ( filename, filename );

	pSkin = CG_FindPSkinSpot (filename);

	if (pSkin->inuse == qtrue)
		return pSkin;

	pSkin->inuse = CG_PSkinUpdateRegistration ( pSkin, filename );

	if ( pSkin->inuse != qtrue )
	{
		if (cg_debugPlayerModels->integer)
			Com_Printf ("pSkin: Registration failed: %s\n", filename);

		memset (pSkin, 0, sizeof(pSkin));
		return NULL;
	}

	return pSkin;
}


//===============
//	CG_PModel_RegisterBoneposes
//	Sets up skeleton with inline boneposes based on frame/oldframe values
//===============
void CG_PModel_RegisterBoneposes( pmodel_t *pmodel )
{
	cgs_skeleton_t		*skel;

	if( !pmodel->pmodelinfo->model )
		return;

	skel = CG_SkeletonForModel( pmodel->pmodelinfo->model );
	if( skel && skel == pmodel->skel )
		return;

	if( !skel ) //not skeletal model
	{
		if( pmodel->skel )
		{
			if( pmodel->curboneposes )	{
				CG_Free( pmodel->curboneposes );
				pmodel->curboneposes = NULL;
			}
			if( pmodel->oldboneposes ) {
				CG_Free( pmodel->oldboneposes );
				pmodel->oldboneposes = NULL;
			}

			pmodel->skel = NULL;
		}
		return;
	}

	if( !pmodel->skel || pmodel->skel && (pmodel->skel->numBones != skel->numBones)
		|| !pmodel->curboneposes || !pmodel->oldboneposes )
	{
		if( pmodel->curboneposes )	
			CG_Free( pmodel->curboneposes );
		if( pmodel->oldboneposes )
			CG_Free( pmodel->oldboneposes );
		
		pmodel->curboneposes = CG_Malloc ( sizeof(bonepose_t) * skel->numBones );
		pmodel->oldboneposes = CG_Malloc ( sizeof(bonepose_t) * skel->numBones );
	}

	pmodel->skel = skel;
}


/*
================
CG_PModelUpdateRegistration
================
*/
qboolean CG_PModelUpdateRegistration ( pmodelinfo_t *pmodelinfo, char *filename )
{
	qboolean 	loaded_model = qfalse;
	char	anim_filename[MAX_QPATH];
	char	scratch[MAX_QPATH];

	Q_snprintfz (scratch, sizeof(scratch), "%s/tris.skm", filename);
	pmodelinfo->model = CG_RegisterModel(scratch);
	if( !trap_R_SkeletalGetNumBones( pmodelinfo->model, NULL ) ) //save the skeleton pointer inside the pmodel
		return qfalse;
	
	//load animations script
	if( pmodelinfo->model ) {
		Q_snprintfz (anim_filename, sizeof(anim_filename), "%s/animation.cfg", filename);
		loaded_model = CG_ParseAnimationScript ( pmodelinfo, anim_filename );
	}
	//clean up if failed
	if (!loaded_model){
		pmodelinfo->model_name[0] = 0;
		pmodelinfo->model = NULL;
		return qfalse;
	}

	strcpy (pmodelinfo->model_name, filename);
	return qtrue;
}


/*
===============
CG_FindPModelSpot
===============
*/
struct pmodelinfo_s *CG_FindPModelSpot ( char *filename )
{
	int 			i;
	int				freespot = -1;

	for (i=0; i<CG_MAX_PMODELS; i++)
	{
		if (cg_PModelInfos[i].inuse == qtrue)
		{
			if (!Q_stricmp (cg_PModelInfos[i].model_name, filename))//found it
			{
				if (cg_debugPlayerModels->integer)
					Com_Printf ("PModel: found at spot %i: %s\n", i, filename);

				return &cg_PModelInfos[i];
			}
		}
		else if (freespot < 0)
			freespot = i;
	}

	if (freespot < 0)
		CG_Error ("CG_FindPModelSpot: Couldn't find a free player model spot");

	if (cg_debugPlayerModels->integer)
		Com_Printf ("PModel: assigned free spot %i for pmodel %s\n", freespot, filename);

	//we have a free spot
	return &cg_PModelInfos[freespot];
}


/*
===============
CG_RegisterPModel

  PModel is not exactly the model, but the indexes of the
  models contained in the pmodel and it's animation data
===============
*/
struct pmodelinfo_s *CG_RegisterPModel ( char *filename )
{
	pmodelinfo_t	*pmodelinfo;

	COM_StripExtension ( filename, filename );

	pmodelinfo = CG_FindPModelSpot (filename);

	if (pmodelinfo->inuse == qtrue)
		return pmodelinfo;

	pmodelinfo->inuse = CG_PModelUpdateRegistration ( pmodelinfo, filename );

	if ( pmodelinfo->inuse != qtrue )
	{
		if (cg_debugPlayerModels->integer)
			Com_Printf ("PModel: Registration failed: %s\n", filename);

		memset (pmodelinfo, 0, sizeof(pmodelinfo));
		return NULL;
	}

	return pmodelinfo;
}


/*
================
CG_RegisterBasePModel

  Default fallback replacements
================
*/
void CG_RegisterBasePModel(void)
{
	char	filename[MAX_QPATH];

	//pmodelinfo
	Q_snprintfz (filename, sizeof(filename), "%s/%s", "models/players", DEFAULT_PLAYERMODEL);
	cgs.basePModelInfo = CG_RegisterPModel ( filename );
	//pskin
	Q_snprintfz (filename, sizeof(filename), "%s/%s/%s","models/players" , DEFAULT_PLAYERMODEL, DEFAULT_PLAYERSKIN);
	cgs.basePSkin = CG_RegisterPSkin (filename);

	//Default player model must load perfect
	if(!cgs.basePModelInfo)
		CG_Error ( "'Default Player Model'(%s): failed to load", DEFAULT_PLAYERMODEL );

	if(!cgs.basePSkin)
		CG_Error ( "'Default Player Model'(%s): Skin (%s) failed to load", DEFAULT_PLAYERMODEL, DEFAULT_PLAYERMODEL );

	if (!cgs.basePSkin->icon)
		CG_Error ( "'Default Player Model'(%s): Icon failed to load", DEFAULT_PLAYERMODEL );
}


/*
================
CG_LoadClientPmodel
Register a special client pmodel (not indexed)
================
*/
void CG_LoadClientPmodel ( int centNum, char *model_name, char *skin_name )
{
	pmodel_t			*pmodel;
	char				filename[MAX_QPATH];
	char				*path = "models/players";
	char				skin_filename[MAX_QPATH];
	
	if (centNum < 0)
		CG_Error("'CG_LoadClientPmodel' wrong client number\n");
	
	pmodel = &cg_entPModels[centNum];
	pmodel->number = centNum;

	//load user defined pmodelinfo
	Q_snprintfz (filename, sizeof(filename), "%s/%s",path, model_name);
	pmodel->pmodelinfo = CG_RegisterPModel( filename );
	//load skin
	Q_snprintfz (skin_filename, sizeof(skin_filename), "%s/%s/%s",path, model_name, skin_name);
	pmodel->pSkin = CG_RegisterPSkin (skin_filename);
	
	
	//didn't work: try DEFAULT_PLAYERMODEL with the user defined skin (for ctf skins)
	if ((!pmodel->pmodelinfo || !pmodel->pSkin) && Q_stricmp(model_name, DEFAULT_PLAYERMODEL))
	{
		strcpy(model_name, DEFAULT_PLAYERMODEL);
		Q_snprintfz (filename, sizeof(filename), "%s/%s",path, model_name);
		pmodel->pmodelinfo = CG_RegisterPModel( filename );
		//load skin
		Q_snprintfz (skin_filename, sizeof(skin_filename), "%s/%s/%s",path, model_name, skin_name);
		pmodel->pSkin = CG_RegisterPSkin (skin_filename);
	}
	
	//if still didn't load, it means that the DEFAULT_PLAYERMODEL
	//didn't have the custom skin so back to DEFAULT_PLAYERSKIN
	if (!pmodel->pSkin)
	{
		strcpy(skin_name, DEFAULT_PLAYERSKIN);
		Q_snprintfz (skin_filename, sizeof(skin_filename), "%s/%s/%s",path, model_name, skin_name);
		pmodel->pSkin = CG_RegisterPSkin (skin_filename);
	}

	
	if (!pmodel->pmodelinfo)
		CG_Error ( "Couldn't load default player model" );

	//Init the boneposes for animation blending
	CG_PModel_RegisterBoneposes( pmodel );
	

	//store weaponIndexes
	CG_LoadPModelWeaponSet ( pmodel, model_name );
	

	//update the frames for the new animation script
	if (!pmodel->anim.current[UPPER] || !pmodel->anim.current[LOWER] || pmodel->anim.current[HEAD] != ANIM_NONE )
	{
		pmodel->anim.current[UPPER] = TORSO_STAND;
		pmodel->anim.current[LOWER] = LEGS_STAND;
		pmodel->anim.current[HEAD] = ANIM_NONE;
	}
	
	pmodel->anim.oldframe[UPPER] = pmodel->anim.frame[UPPER] = pmodel->pmodelinfo->lastframe[pmodel->anim.current[UPPER]];
	pmodel->anim.oldframe[LOWER] = pmodel->anim.frame[LOWER] = pmodel->pmodelinfo->lastframe[pmodel->anim.current[LOWER]];
	pmodel->anim.oldframe[HEAD] = pmodel->anim.frame[HEAD] = pmodel->pmodelinfo->lastframe[pmodel->anim.current[HEAD]];
}



//======================================================================
//							tools
//======================================================================



/*
===============
CG_Pmodel_TagMask
===============
*/
char *CG_Pmodel_TagMask( char *maskname, pmodelinfo_t *pmodelinfo )
{
	cg_tagmask_t	*tagmask;

	if( !pmodelinfo )
		return maskname;

	for( tagmask = pmodelinfo->tagmasks; tagmask; tagmask = tagmask->next ) {
		if( !Q_stricmp( tagmask->tagname, maskname ) )
			return tagmask->bonename;
	}

	return maskname;
}

/*
===============
CG_GrabTag
 In the case of skeletal models, boneposes must
 be transformed prior to calling this function
===============
*/
qboolean CG_GrabTag( orientation_t *tag, entity_t *ent, char *tagname )
{
	cgs_skeleton_t	*skel;
	
	if (!ent->model)
		return qfalse;
	
	skel = CG_SkeletonForModel( ent->model );
	if( skel )
		return CG_SkeletalPoseGetAttachment( tag, skel, ent->boneposes, tagname );
	
	return trap_R_LerpTag( tag, ent->model, ent->frame, ent->oldframe, ent->backlerp, tagname );
}


/*
===============
CG_PlaceRotatedModelOnTag
===============
*/
void CG_PlaceRotatedModelOnTag (entity_t *ent, entity_t *dest, orientation_t *tag)
{
	int		i;
	vec3_t	tmpAxis[3];

	VectorCopy( dest->origin, ent->origin );
	VectorCopy (dest->lightingOrigin, ent->lightingOrigin);
	
	for ( i = 0 ; i < 3 ; i++ )
		VectorMA( ent->origin, tag->origin[i], dest->axis[i], ent->origin );

	VectorCopy (ent->origin, ent->oldorigin);

	Matrix_Multiply( ent->axis, tag->axis, tmpAxis );
	Matrix_Multiply( tmpAxis, dest->axis, ent->axis );
}


/*
===============
CG_PlaceModelOnTag
===============
*/
void CG_PlaceModelOnTag (entity_t *ent, entity_t *dest, orientation_t *tag)
{
	int i;
	
	VectorCopy (dest->origin, ent->origin);
	VectorCopy (dest->lightingOrigin, ent->lightingOrigin);
	
	for ( i = 0 ; i < 3 ; i++ )
		VectorMA( ent->origin, tag->origin[i], dest->axis[i], ent->origin );
	
	VectorCopy (ent->origin, ent->oldorigin);
	
	Matrix_Multiply( tag->axis, dest->axis, ent->axis );
}

/*
//
//===============
//CG_MoveToTag
//===============
//
void CG_MoveToTag ( orientation_t *result, orientation_t *ref, orientation_t *tag)
{
	int i;
	
	VectorCopy (ref->origin, result->origin);
	
	for ( i = 0 ; i < 3 ; i++ )
		VectorMA( result->origin, tag->origin[i], ref->axis[i], result->origin );
	
	Matrix_Multiply( tag->axis, ref->axis, result->axis );
}

//
//===============
//CG_PModel_CalcProjectionSource
//===============
//
qboolean CG_PModel_CalcProjectionSource( pmodel_t *pmodel, orientation_t *projection )
{
	orientation_t tag_weapon;
	orientation_t	result1, ref;

	if(!pmodel->pmodelinfo)
		return qfalse;

	if( !pmodel->ent.model )
		return qfalse;

	//the code asumes that the pmodel ents are up-to-date
	if(!trap_R_LerpTag( &tag_weapon, pmodel->ent.model, pmodel->anim.frame[UPPER], pmodel->anim.oldframe[UPPER], pmodel->ent.backlerp, "tag_weapon" ) )
		return qfalse;

	VectorCopy( pmodel->ent.origin, ref.origin );
	Matrix_Copy( pmodel->ent.axis, ref.axis );
	CG_MoveToTag ( &result1, &ref, &tag_weapon );

	//our origin is now at tag_weapon. Move it to tag_projectionsource
	CG_MoveToTag ( projection, &result1, &pmodel->pweapon.weaponInfo->tag_projectionsource );
	return qtrue;
}

//
//===============
//CG_PModel_CalcBrassSource
//===============
//
void CG_PModel_CalcBrassSource( pmodel_t *pmodel, orientation_t *projection )
{
	orientation_t	tag_weapon;
	orientation_t	ref;

	//the code asumes that the pmodel ents are up-to-date
	if(!pmodel->pmodelinfo || !pmodel->ent.model || !pmodel->ent.model 
		|| !trap_R_LerpTag( &tag_weapon, pmodel->ents[UPPER].model, pmodel->anim.frame[UPPER], pmodel->anim.oldframe[UPPER], pmodel->ent.backlerp, "tag_weapon" ) )
	{
		VectorCopy(pmodel->ents[LOWER].origin, projection->origin);
		projection->origin[2] += 16;
		Matrix_Identity(projection->axis);
	}
	//brass projection is simply tag_weapon
	VectorCopy( pmodel->ents[UPPER].origin, ref.origin );
	Matrix_Copy( pmodel->ents[UPPER].axis, ref.axis );
	CG_MoveToTag ( projection, &ref, &tag_weapon );
	//projection->origin[2] += 8;
}
*/

/*
===============
CG_AddShellEffects
===============
*/
void CG_AddQuadShell(entity_t *ent)
{
	entity_t	shell;

	shell = *ent;
	shell.customSkin = NULL;

	if ( shell.flags & RF_WEAPONMODEL )
		shell.customShader = CG_MediaShader ( cgs.media.shaderQuadWeapon );
	else
		shell.customShader = CG_MediaShader ( cgs.media.shaderPowerupQuad );

	CG_AddEntityToScene( &shell );//vicskmblend
}


void CG_AddPentShell (entity_t *ent)
{
	entity_t	shell;

	shell = *ent;
	shell.customSkin = NULL;

	if ( shell.flags & RF_WEAPONMODEL )
		shell.customShader = CG_MediaShader ( cgs.media.shaderPowerupPenta );//fixme: look for a PENTA shader for view weapon
	else
		shell.customShader = CG_MediaShader ( cgs.media.shaderPowerupPenta );

	CG_AddEntityToScene( &shell );//vicskmblend
}


void CG_AddShellEffects (entity_t *ent, int effects)
{
	// quad and pent can do different things on client

	if (effects & EF_QUAD) 
		CG_AddQuadShell (ent);

	if (effects & EF_PENT) 
		CG_AddPentShell (ent);
}

/*
===============
CG_AddColorShell
===============
*/
void CG_AddColorShell_Do (entity_t *ent, int renderfx)
{
	entity_t	shell;
	int			i;
	vec4_t shadelight = { 0, 0, 0, 0.3 };

	memset (&shell, 0, sizeof(shell));
	
	shell = *ent;
	shell.customSkin = NULL;
	
	if ( renderfx & RF_SHELL_RED )
		shadelight[0] = 1.0;
	if ( renderfx & RF_SHELL_GREEN )
		shadelight[1] = 1.0;
	if ( renderfx & RF_SHELL_BLUE )
		shadelight[2] = 1.0;
	
	for (i=0 ; i<4 ; i++)
		shell.color[i] = shadelight[i] * 255;
	
	if ( ent->flags & RF_WEAPONMODEL )
		return;//fixme: try the shell shader for viewweapon, or build a good one
	else
		shell.customShader = CG_MediaShader ( cgs.media.shaderShellEffect );

	CG_AddEntityToScene( &shell );//vicskmblend
}

void CG_AddColorShell (entity_t *ent, int renderfx)
{
	if ( !(renderfx & (RF_SHELL_GREEN | RF_SHELL_RED | RF_SHELL_BLUE)) )
		return;

	CG_AddColorShell_Do (ent, renderfx);
}


//======================================================================
//							animations
//======================================================================


//===============
// CG_PModel_BlendSkeletalPoses
// Sets up the boneposes array mixing up bones
// from different frames, based on the defined
// bone names. Result bones are not transformed.
//===============
void CG_PModel_BlendSkeletalPoses(cgs_skeleton_t *skel, bonepose_t *boneposes, int numBoneKeys, unsigned char *boneKeys, int *boneKeyFrames)
{
	int				i, j;
	bonepose_t		*framed_bonepose;

	for( j = 0; j < skel->numBones; j++ ) 
	{
		//find bonepose in frame
		framed_bonepose = skel->bonePoses[boneKeyFrames[0]];//first bonekey in chain for unrecognized BoneKey tokens
		for( i=0; i<numBoneKeys; i++ )
		{
			if( skel->bones[j].name[0] == boneKeys[i]){
				framed_bonepose = skel->bonePoses[boneKeyFrames[i]];
				break;
			}
		}
		framed_bonepose += j;
		memcpy(&boneposes[j], framed_bonepose, sizeof(bonepose_t));
	}
}


/*
===============
CG_PModelAnimToFrame

  Transforms the values inside state->frame into 3 animation values
  for head, torso and legs (head is always zero right now). It also
  uses those values to handle the animation frames clientside, as 
  also does the backlerp calculation for variable framerates.
  
  The frames and other data is stored inside animationinfo_t. Actually
  only clientinfo has structs of this type inside, but they could also
  be applied to a client-side monsterinfo handler.
  
  It works like this:
  -for animations reveived by BASIC_CHANNEL:
  When a new animation is recieved, the new animation is launched.
  While 0 is recieved, it advances the frames in last recieved animation.
  When the frame gets to the end of the animation, it looks for the
  looping value and sends the frame back to (lastframe - loopingframes)
  and so on. If the looping value is 0, it holds the animation in the
  last frame until a new animation is recieved.

  -for animations reveived by EVENT_CHANNEL:
  The playing BASIC_CHANNEL animation is put back on the animation
  buffer, the EVENT_CHANNEL animation is played, and when it comes to 
  it's end,the BASIC_CHANNEL animation at buffer (the same as we put in, 
  or a new one received by that time) is promoted to be played.
===============
*/
void CG_PModelAnimToFrame (pmodel_t *pmodel, animationinfo_t *anim)
{
	int i;
	pmodelinfo_t	*pmodelinfo = pmodel->pmodelinfo;
	cgs_skeleton_t	*skel;

	//interpolate
	if (cg.time < anim->nextframetime)
	{
		anim->backlerp = 1.0f - ((cg.time - anim->prevframetime)/(anim->nextframetime - anim->prevframetime));
		if (anim->backlerp > 1)
			anim->backlerp = 1;
		else if (anim->backlerp < 0)
			anim->backlerp = 0;
		
		return;
	}
	
	for (i=0; i<PMODEL_PARTS; i++)
	{
		//advance frames
		anim->oldframe[i] = anim->frame[i];
		anim->frame[i]++;
		
		//looping
		if (anim->frame[i] > pmodelinfo->lastframe[anim->current[i]])
		{
			if (anim->currentChannel[i])	
				anim->currentChannel[i] = BASIC_CHANNEL;//kill

			anim->frame[i] = (pmodelinfo->lastframe[anim->current[i]] - pmodelinfo->loopingframes[anim->current[i]]);
		}

		//NEWANIM
		if ( anim->buffer[EVENT_CHANNEL].newanim[i] )
		{
			//backup if basic
			if (anim->buffer[BASIC_CHANNEL].newanim[i] +
				anim->currentChannel[i] == BASIC_CHANNEL){//both are zero
				anim->buffer[BASIC_CHANNEL].newanim[i] = anim->current[i];
			}
			//set up
			anim->current[i] = anim->buffer[EVENT_CHANNEL].newanim[i];
			anim->frame[i] = pmodelinfo->firstframe[anim->current[i]];
			anim->currentChannel[i] = EVENT_CHANNEL;
			anim->buffer[EVENT_CHANNEL].newanim[i] = 0;
		}
		else if ( anim->buffer[BASIC_CHANNEL].newanim[i]
			&& ( anim->currentChannel[i] != EVENT_CHANNEL ) )
		{
			//set up
			anim->current[i] = anim->buffer[BASIC_CHANNEL].newanim[i];
			anim->frame[i] = pmodelinfo->firstframe[anim->current[i]];
			anim->currentChannel[i] = BASIC_CHANNEL;
			anim->buffer[BASIC_CHANNEL].newanim[i] = 0;
		}
	}

	//new method
	skel = CG_SkeletonForModel( pmodel->pmodelinfo->model );
	if(!skel)
		CG_Error("Non-skeletal PModel inside 'CG_PModelAnimToFrame'\n");

	if( skel != pmodel->skel )
		CG_PModel_RegisterBoneposes( pmodel ); //update registration

	//blend animations in poses
	CG_PModel_BlendSkeletalPoses( skel, pmodel->curboneposes, pmodel->pmodelinfo->numbonekeys, pmodel->pmodelinfo->boneKeys, pmodel->anim.frame);
	CG_PModel_BlendSkeletalPoses( skel, pmodel->oldboneposes, pmodel->pmodelinfo->numbonekeys, pmodel->pmodelinfo->boneKeys, pmodel->anim.oldframe);

	//updated frametime
	anim->prevframetime = cg.time;
	anim->nextframetime = cg.time + pmodelinfo->frametime;
	anim->backlerp = 1.0f;
}


/*
===============
CG_ClearEventAnimations

  Called from CG_PModelUpdateState if the new state has a EV_TELEPORT.
  This is done so respawning can reset animations, because event
  animations could persist after dying
===============
*/
void CG_ClearEventAnimations(entity_state_t *state)
{
	int i;
	pmodel_t	*pmodel;

	pmodel = &cg_entPModels[state->number];

	for ( i = LOWER ; i < PMODEL_PARTS ; i++ )
	{
		//clean up the buffer
		pmodel->anim.buffer[EVENT_CHANNEL].newanim[i] = 0;

		//finish up currents
		if (pmodel->anim.currentChannel[i] == EVENT_CHANNEL)
		{
			pmodel->anim.frame[i] = pmodel->pmodelinfo->lastframe[pmodel->anim.current[i]];
			pmodel->anim.currentChannel[i] = BASIC_CHANNEL;
		}
	}
}


/*
===============
CG_AddPModelAnimation
===============
*/
void CG_AddPModelAnimation ( pmodel_t *pmodel, int loweranim, int upperanim, int headanim, int channel)
{
	int	i;
	int newanim[PMODEL_PARTS];
	animationbuffer_t	*buffer;

	newanim[LOWER] = loweranim;
	newanim[UPPER] = upperanim;
	newanim[HEAD] = headanim;

	buffer = &pmodel->anim.buffer[channel];

	for ( i = 0 ; i < PMODEL_PARTS ; i++ )
	{
		//ignore new events if in death
		if (channel && buffer->newanim[i] && (buffer->newanim[i] < TORSO_GESTURE))
			continue;

		if (newanim[i] && (newanim[i] < PMODEL_MAX_ANIMS))
		{
			buffer->newanim[i] = newanim[i];
		}
	}
}

/*
===============
CG_AddAnimationFromState
===============
*/
void CG_AddAnimationFromState ( entity_state_t *state, int loweranim, int upperanim, int headanim, int channel)
{
	CG_AddPModelAnimation ( &cg_entPModels[state->number], loweranim, upperanim, headanim, channel);
}


/*
==============
CG_PModels_AddFireEffects

  launch fire animations from events
==============
*/
void CG_PModels_AddFireEffects ( entity_state_t *state, int parms )
{
	pmodel_t *pmodel;

	if ( state->effects & EF_CORPSE )
		return;

	pmodel = &cg_entPModels[state->number];

	//activate weapon flash
	if (cg_weaponFlashes->integer)	//splitskel
		pmodel->pweapon.flashtime = cg.time + (int)((pmodel->pmodelinfo->frametime/4)*3);


	//all them are the same, but still...
	switch ( state->weapon )
	{
	case WEAP_BLASTER:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_HYPERBLASTER:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_MACHINEGUN:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_SHOTGUN:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_SUPERSHOTGUN:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_CHAINGUN:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_RAILGUN:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_ROCKETLAUNCHER:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_GRENADELAUNCHER:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	case WEAP_BFG:
		CG_AddPModelAnimation ( pmodel, 0, TORSO_ATTACK1, 0, EVENT_CHANNEL);
		break;
		
	default:
		break;
	}
}


//======================================================================
//							player model
//======================================================================


/*
==================
CG_PModelUpdateState

  Called each new serverframe
==================
*/
void CG_PModelUpdateState (entity_state_t *state)
{
	int newanim[PMODEL_PARTS];
	entity_state_t *oldstate;
	int			i;
	pmodel_t	*pmodel;

	pmodel = &cg_entPModels[state->number];
	
	if (state->modelindex != 255)	 
	{
		//in ctf, corpses mirror it's client pmodel (ctf skins)
		if( cgs.gametype == GAMETYPE_CTF && state->effects & EF_CORPSE ) {
			pmodel->pmodelinfo = cg_entPModels[(state->skinnum & 0xff)+1].pmodelinfo;
			pmodel->pSkin = cg_entPModels[(state->skinnum & 0xff)+1].pSkin;
		} else {	//indexed
			pmodel->pmodelinfo = cgs.pModelsIndex[state->modelindex];
			pmodel->pSkin = cgs.pSkinsIndex[state->skinnum];
		}
	}

	//fallback
	if (!pmodel->pmodelinfo || !pmodel->pSkin){
		pmodel->pmodelinfo = cgs.basePModelInfo;
		pmodel->pSkin = cgs.basePSkin;
	}

	CG_PModel_RegisterBoneposes( pmodel );//jalfixme: I'm doing this 3 times, cut it
	
	//Update pweapon
	pmodel->pweapon.weaponInfo = CG_GetWeaponFromPModelIndex( pmodel, state->weapon );

	//vicskmblend: register boneposes for cg_entity
	CG_RegisterBoneposesForCGEntity( &cg_entities[pmodel->number], pmodel->pmodelinfo->model );

	//update parts rotation angles
	for ( i = LOWER ; i < PMODEL_PARTS ; i++ )
		VectorCopy(pmodel->angles[i], pmodel->oldangles[i]);
	
	if(!(state->effects & EF_CORPSE))
	{
		//lower has horizontal direction, and zeroes vertical
		pmodel->angles[LOWER][PITCH] = 0;
		pmodel->angles[LOWER][YAW] = state->angles[YAW];
		pmodel->angles[LOWER][ROLL] = 0;

		//upper marks vertical direction (total angle, so it fits aim)
		if (state->angles[PITCH] > 180)
			pmodel->angles[UPPER][PITCH] = (-360 + state->angles[PITCH]);
		else
			pmodel->angles[UPPER][PITCH] = state->angles[PITCH];
		
		pmodel->angles[UPPER][YAW] = 0;
		pmodel->angles[UPPER][ROLL] = 0;
		
		//head adds a fraction of vertical angle again
		if (state->angles[PITCH] > 180)
			pmodel->angles[HEAD][PITCH] = (-360 + state->angles[PITCH])/3;
		else
			pmodel->angles[HEAD][PITCH] = state->angles[PITCH]/3;
		
		pmodel->angles[HEAD][YAW] = 0;
		pmodel->angles[HEAD][ROLL] = 0;
	}

	//Spawning (EV_TELEPORT) forces nobacklerp and the interruption of EVENT_CHANNEL animations
	if ((state->events[0] == EV_TELEPORT) || (state->events[1] == EV_TELEPORT))
	{
		CG_ClearEventAnimations(state);
		CG_AddAnimationFromState ( state, (state->frame)&0x3F, (state->frame>>6)&0x3F, (state->frame>>12)&0xF, BASIC_CHANNEL);//(jalPVSfix2)
		//set up old frame
		pmodel->anim.nextframetime = cg.time;

		CG_PModelAnimToFrame( pmodel, &pmodel->anim );
		pmodel->anim.oldframe[LOWER] = pmodel->anim.frame[LOWER];
		pmodel->anim.oldframe[UPPER] = pmodel->anim.frame[UPPER];
		pmodel->anim.oldframe[HEAD] = pmodel->anim.frame[HEAD];

		for ( i = LOWER ; i < PMODEL_PARTS ; i++ )
			VectorCopy(pmodel->angles[i], pmodel->oldangles[i]);

		return;
	}

	oldstate = &cg_entities[state->number].prev;
	
	//filter repeated animations coming from state->frame
	newanim[LOWER] = (state->frame&0x3F) * ((state->frame &0x3F) != (oldstate->frame &0x3F));
	newanim[UPPER] = (state->frame>>6 &0x3F) * ((state->frame>>6 &0x3F) != (oldstate->frame>>6 &0x3F));
	newanim[HEAD] = (state->frame>>12 &0xF) * ((state->frame>>12 &0xF) != (oldstate->frame>>12 &0xF));
	
	CG_AddAnimationFromState ( state, newanim[LOWER], newanim[UPPER], newanim[HEAD], BASIC_CHANNEL);
}


/*
==================
CG_PModelsUpdateStates

  Called each new serverframe
==================
*/
void CG_PModelsUpdateStates (void)
{
	int					pnum;
	entity_state_t		*state;

	for ( pnum = 0; pnum < cg.frame.numEntities; pnum++ )
	{
		state = &cg_parseEntities[(cg.frame.parseEntities+pnum)&(MAX_PARSE_ENTITIES-1)];
		if ( (state->type & ~ET_INVERSE) == ET_PLAYER ) {
			CG_PModelUpdateState (state);
		}
	}
}

/*
===============
CG_AddPModelEnt

===============
*/
void CG_AddPModel (entity_t *ent, entity_state_t *state)
{
	int					i, j;
	pmodel_t			*pmodel;
	vec3_t				tmpangles;
	orientation_t		tag_weapon;
	
	pmodel = &cg_entPModels[state->number];
	
	//transform animation values into frames
	CG_PModelAnimToFrame( pmodel, &pmodel->anim );
	
	//relink boneposes
	ent->boneposes = ent->oldboneposes = centBoneposes[state->number].lerpboneposes;

	//new method
	CG_LerpBoneposes( pmodel->skel,
		pmodel->curboneposes, pmodel->oldboneposes,
		centBoneposes[state->number].lerpboneposes,
		1.0f - pmodel->anim.backlerp );

	//add effects
	if( state->effects & EF_CORPSE ) {
		AnglesToAxis ( state->angles, ent->axis );

	} else {
		
		//apply lerped LOWER angles to entity
		for ( j = 0; j < 3; j++ )
			tmpangles[j] = LerpAngle ( pmodel->oldangles[LOWER][j], pmodel->angles[LOWER][j], cg.lerpfrac );
		
		AnglesToAxis ( tmpangles, ent->axis );
		
		//apply UPPER and HEAD angles to rotator bones
		for (i=1; i<PMODEL_PARTS; i++)	
		{
			if(pmodel->pmodelinfo->numRotators[i])
			{
				//lerp rotation and divide angles by the number of rotation bones
				for ( j = 0; j < 3; j++ ){
					tmpangles[j] = LerpAngle ( pmodel->oldangles[i][j], pmodel->angles[i][j], cg.lerpfrac );
					tmpangles[j] /= pmodel->pmodelinfo->numRotators[i];
				}
				for ( j=0; j<pmodel->pmodelinfo->numRotators[i]; j++)
					CG_RotateBonePose( tmpangles, &ent->boneposes[pmodel->pmodelinfo->rotator[i][j]] );
			}
		}
	}

	//finish
	CG_TransformBoneposes( centBoneposes[state->number].skel, centBoneposes[state->number].lerpboneposes, centBoneposes[state->number].lerpboneposes);


	ent->backlerp = pmodel->anim.backlerp;
	//ent->backlerp = 0.0f;
	ent->frame = ent->oldframe = 0;//frame fields are not used with external poses

	// Add playermodel ent
	ent->scale = 1.0f;
	ent->rtype = RT_MODEL;
	ent->customShader = NULL;
	ent->model = pmodel->pmodelinfo->model;
	ent->customSkin = pmodel->pSkin->skin;

	
	CG_AddEntityToScene( ent );
	if( !ent->model )
		return;

	CG_AddShellEffects ( ent, state->effects );
	CG_AddColorShell ( ent, state->renderfx );
	

	// Add weapon model
	if (state->modelindex2 != 255) //player doesn't have a weapon
		return;

	if( CG_GrabTag( &tag_weapon, ent, CG_Pmodel_TagMask("tag_weapon", pmodel->pmodelinfo) ) )
		CG_AddWeaponOnTag ( ent, &tag_weapon, &pmodel->pweapon, 0);
}

